/*********************************************************************
GantSonar v1_2
Dispositif haptique portatif pour la perception de l'environnement proche par des mal-voyants
Matériel utilisé : Arduino UNO ou NANO + (2 x HC-SR04) + LD293D + (2 x vibreurs)
Principe : l'écho des objets détectés est rendu par une vibration inversement proportionnelle à leur distance
Développement : Yves Le Chevalier (idée de Steve Hoefer) pour le LABFAB de Rennes - Mars 2016
NB : aucune librairie n'est requise pour ce programme (compatibilité avec toutes versions IDE)
*********************************************************************/
#define TRACES 0   // mise au point : 1 pour activer trace sur port série, 0 pour désactiver
#define BUZZER 0   // aide aux réglages : 1 pour activer le buzzer,  0 pour le désactiver
//------------------------------------------------------------------

// BUZZER d'aide aux réglages des paramètres de vibrations (tests sur sonar+vibreur 1)
#define pbuzz 14   // pin D14 (=A0) : génération fréquence sonore inv.prop. à distance 

// LATERALITE DE L'UTILISATEUR
#define gaucher 0    //  1 pour un gaucher,   0 pour un droitier (par défaut)

// VARIABLES DISTANCES
#define bdismax 2   // pin D2 : bouton mémorisation distance maxi (interruption 0 sur pin D2)
#define distmin 5  // distance mini de détection correspondant à une vibration maxi (5cm par défaut)
#define distmaxima 250 // distance maxi de capacité de détection (selon sonar) 
int distmax = distmaxima;  // distance maxi de détection réglée par l'utilisateur (bouton)
int dist = 0;       // variable de calcul distance
unsigned long timpre, timbou = 0; // variable de temps pour éviter le rebond sur appui bouton

// VARIABLES FORCES DE VIBRATIONS
#define vibmini 10  // vibration minimale perceptible   (à ajuster selon le vibreur)
#define vibmaxi 150 // vibration maximale (à ajuster selon le vibreur pour le confort de l'usager)
#define fordem 50  // force minimale pour démarrage des vibreurs
int forvib = 0;    // variable de calcul force de vibration

// VIBRATIONS EN MODE PULSE OU EN CONTINU
int freqvib1 = 200; // durée vibrations et repos pour un signal pulsé ; sinon 0 pour signal continu (millisec)
int freqvib2 = 200; // durée vibrations et repos pour un signal pulsé ; sinon 0 pour signal continu (millisec)

// SONAR 1   (Sonar du côté gauche vu de dessus)
#define ptrig1 11   //  pin D11 : trigger d'activation dur sonar 1     
#define pecho1 6    //  pin D6 : renvoi une durée d'écho proportionnelle à la distance
int dist1;          // distance mesurée par sonar 1

// SONAR 2   (Sonar du côté droit vu de dessus)
#define ptrig2 12   //  pin D12 : trigger d'activation dur sonar 2 
#define pecho2 7  //  pin D7 : renvoi une durée d'écho proportionnelle à la distance
int dist2;          // distance mesurée par sonar 2

// VIBREUR 1   (Vibreur du côté gauche vu de dessus)
#define penab1 5    // pin D5 : commande vitesse vibreur 1 (pin 1 du L293D)
#define pinpu11 4   // pin D4 : commande 1 sens de rotation (pin 2 du L293D)
#define pinpu12 3   // pin D3 : commande 2 sens de rotation (pin 7 du L293D)
unsigned long timvib1 = 0;  // comptage temps de vibration
unsigned long timrepo1 = 0;  // comptage temps de repos
boolean indvib1 = false;   //  indicateur vibration en cours
int valvib1;          // force de vibration à appliquer

// VIBREUR 2    (Vibreur du côté droit vu de dessus)
#define penab2 10   // pin D10 : commande vitesse vibreur 2 (pin 9 du L293D)
#define pinpu21 9  // pin D9 : commande 1 sens de rotation (pin 10 du L293D)
#define pinpu22 8   // pin D8 : commande 2 sens de rotation (pin 15 du L293D)
unsigned long timvib2 = 0;  // comptage temps de vibration
unsigned long timrepo2 = 0;  // comptage temps de repos
boolean indvib2 = false;   //  indicateur vibration en cours
int valvib2;            // force de vibration à appliquer

// ***************************** SETUP *******************
void setup() {
#if TRACES        // Trace pour la mise au point du programme sur port série
  Serial.begin(9600);
  Serial.println("Gant sonar - Mise au point");
#endif
  pinMode(ptrig1, OUTPUT);     // initialisation du sens des pins de la carte
  pinMode(ptrig2, OUTPUT);
  pinMode(pecho1, INPUT);
  pinMode(pecho2, INPUT);
  pinMode(penab1, OUTPUT);
  pinMode(pinpu11, OUTPUT);
  pinMode(pinpu12, OUTPUT);
  pinMode(penab2, OUTPUT);
  pinMode(pinpu21, OUTPUT);
  pinMode(pinpu22, OUTPUT);
  #if BUZZER
    pinMode(pbuzz, OUTPUT);      // Buzzer pour réglages des paramètres de force de vibrations
  #endif
  digitalWrite(pinpu11, LOW);   // pin 1 low + pin 2 high ...
  digitalWrite(pinpu12, HIGH);  //    ... vibreur 1 tournera dans sens horaire
  analogWrite(penab1, 0);       // vibreur 1 arrêté au départ
  digitalWrite(pinpu21, HIGH);  // pin 1 high + pin 2 low ...
  digitalWrite(pinpu22, LOW);   //    ... vibreur 2 tournera dans sens anti-horaire
  analogWrite(penab2, 0);       // vibreur 2 arrêté au départ
  // NB : on fait tourner les deux vibreurs en sens inverse l'un de l'autre afin d'avoir
  //      une homothétie des vibrations puisque les vibreur sont disposés de chaque côté de la main.
  attachInterrupt(0, memodist, RISING );   // bouton mémorisation distance mini sur broche 2 (interruption #0)
}
// ***************************** MAIN LOOP *******************
void loop() {
  dist1 = mesuredist1();    // mesure echo sur sonar 1
  dist1 = constrain(dist1, distmin, distmaxima + 1);   // limite la distance mesurée entre mini et maxi.
  dist2 = mesuredist2();    // mesure echo sur sonar 2
  dist2 = constrain(dist2, distmin, distmaxima + 1);   // limite la distance mesurée entre mini et maxi.
  #if TRACES
    Serial.print(millis() / 1000);
    Serial.print(" dist1 : ");
    Serial.print(dist1);
    Serial.print(" cm     dist2 : ");
    Serial.print(dist2);
    Serial.print(" cm   distmax : ");
    Serial.print(distmax);
    Serial.print(" cm   freqvib1 : ");
    Serial.print(freqvib1);
    Serial.print(" force1 = ");
    Serial.print(valvib1);
    Serial.print("  freqvib2 : ");
    Serial.print(freqvib2);
    Serial.print(" force2 = ");
    Serial.print(valvib2);
    Serial.print(" indvib : ");
    Serial.print(indvib1);
    Serial.print("  ");
    Serial.println(indvib2);  
  #endif   
  if (dist1 <= distmax) activib1();     // si obstacle détecté : activation vibrations 
  else if (freqvib1 == 0 ) arretvib1();  // sinon : si mode non pulsé : arret immédiat vibration
  if (dist2 <= distmax) activib2();     // si obstacle détecté : activation vibrations 
  else if (freqvib2 == 0)  arretvib2();  // sinon : si mode non pulsé : arret immédiat vibration
  if (freqvib1 > 0 && indvib1 == true) {       // si vibreur 1 en activité
    if (millis() - timvib1 > freqvib1)  arretvib1();  // arret si durée supérieurs à durée prévue
  }  
  if (freqvib2 > 0 && indvib2 == true) {       // si vibreur 2 en activité
    if (millis() - timvib2 > freqvib2)  arretvib2();  // arret si durée supérieurs à durée prévue
  }
}
//------------------------------------------------------
void arretvib1() {  // routine d'arrêt du vibreur 1
  if (indvib1 == true) {       // si vibreur en activité
    analogWrite(penab1, 0);    // arrêt du vibreur 1
    digitalWrite(pinpu12, LOW);  // frein moteur sur vibreur 1 (pinpu11 déjà à LOW) sur vibreur 1 
    timrepo1 = millis();    // initialisation comptage durée de repos
    indvib1 = false;      // le vibreur 1 ne vibre plus
  }
}
void arretvib2() {  // routine d'arrêt du vibreur 2
  if (indvib2 == true) {      // si vibreur en activité
    analogWrite(penab2, 0);   // arrêt du vibreur 2
    digitalWrite(pinpu22, HIGH);  // frein moteur sur vibreur 2 (pinpu21 déjà à HIGH) sur vibreur 2 
    timrepo2 = millis();    // initialisation comptage durée de repos
    indvib2 = false;      // le vibreur 2 ne vibre plus
  }
}
//------------------------------------------------------
int mesuredist1() {    // mesure distance sur sonar 1
  digitalWrite(ptrig1, HIGH);   // initialisation sonar 1
  delayMicroseconds(10);      // envoi de 5v pendant 10 microsec
  digitalWrite(ptrig1, LOW);  // arrêt initialisation
  dist = pulseIn(pecho1, HIGH) / 58;  // lecture echo et conversion distance en centimètres
  return dist;
}
int mesuredist2() {    // mesure distance sur sonar 2
  digitalWrite(ptrig2, HIGH);   // initialisation sonar 2
  delayMicroseconds(1000);       // envoi de 5v pendant 10 microsec
  digitalWrite(ptrig2, LOW);   // arrêt initialisation
  dist = pulseIn(pecho2, HIGH) / 58;  // lecture echo et conversion distance en centimètres
  return dist;
}
//------------------------------------------------------
void memodist() {   // routine d'interruption pour mémoriser la distance maxi mesurée
  timbou = millis();        // memorisation temps actuel
  if (timbou - timpre > 500) {    // temporisation 1/2 sec anti-rebond du bouton
    if (gaucher) dist = mesuredist2();  // pour les gauchers le sonar côté du pouce est celui de droite
    else dist = mesuredist1();         // pour les droitiers le sonar côté du pouce est celui de gauche
    if (dist > 50 || dist < distmin)  distmax = distmaxima;   // remise à distance maxi par défaut
    else distmax = dist * 3;    // sinon la distance maxi est maintenant 3 fois la distance mesurée
    timpre = timbou;       // mémorisation de l'heure d'appui sur le bouton 
  }
}
//------------------------------------------------------
void activib1() {  // routine d'activation du vibreur 1
  if (indvib1 == false || freqvib1 == 0) {      // si vibreur 1 à l'arrêt ou si mode non pulsé 
    if ((freqvib1 == 0) || (millis() - timrepo1 > freqvib1)) {   // vibreur activé si durée repos dépassée ou mode non pulsé
      valvib1 = calcforce(dist1);    // calcul force vibration pour la distance mesurée
      freqvib1 = map(dist1,distmin,distmax,10,200);  // frequence des pulses inversement proportionnelle a distance
      if (indvib1 == false) {      // démarrage moteur que si vibreur est à l'arrêt     
        digitalWrite(pinpu12, HIGH);  // actualiser sens rotation du vibreur 1 (sens horaire)
        analogWrite(penab1, fordem);  // démarrage vibreur 1 (décollage du moteur)
        delay(20);                // applique force de démarrage pendant 50 millisec
      }
      analogWrite(penab1, valvib1);  // activation proportionnelle du vibreur 1
      timvib1 = millis();    // mémo heure de démarrage du vibreur 1
      indvib1 = true;        // vibreur 1 en marche
      #if BUZZER
        if (dist1 > 0) { // calcul fréquence sonore selon distance mesurée (ramenée à dist. maxi)
          int son = map(dist1* (distmaxima / distmax),1,250,1200,150);  // plage fréquences 150 à 1200 Hz  
          tone(pbuzz,son,50);           // générer le son pendant 20 millisec.
        }
      #endif      
    }
  }
}
void activib2() {  // routine d'activation du vibreur 2
  if (indvib2 == false || freqvib2 == 0) {    // si vibreur 2 à l'arrêt ou si mode non pulsé 
    if ((freqvib2 == 0) || (millis() - timrepo2 > freqvib2)) {  // vibreur activé si durée repos dépassée ou mode non pulsé
      valvib2 = calcforce(dist2);    // calcul force vibration pour la distance mesurée
      freqvib2 = map(dist2,distmin,distmax,10,200);  // frequence des pulses inversement proportionnelle a distance
      if (indvib2 == false) {      // démarrage moteur que si vibreur est à l'arrêt
        digitalWrite(pinpu22, LOW);   // actualiser sens rotation du vibreur 2 (anti-horaire)
        analogWrite(penab2, fordem);  // démarrage vibreur 2  (décollage du moteur)        
        delay(20);               // applique force de démarrage pendant 20 millisec
      }
      analogWrite(penab2, valvib2);  // activation proportionnelle du vibreur 2
      timvib2 = millis();     // mémo heure de démarrage du vibreur 2
      indvib2 = true;        // vibreur 2 en marche
    }  
  }
}
//------------------------------------------------------
int calcforce(int dist) {    // calcul force de vibration en fonction inverse de la distance (quelque soit distance maxi)
  int disconv = dist * (distmaxima / distmax);  // Convertir distance réelle mesurée dans l'intervalle de mesure maxi (0 à 250 cm)
  forvib = vibmaxi - (pow(disconv,3)*0.000016);   // calcul de la force de vibration (fonction exponentielle cubique inverse de la distance) 
  if (forvib > 0) constrain(forvib, vibmini, vibmaxi);  // limiter la force de vibration dans l'intervalle mini à maxi voulus 
  else forvib = 0;    // car en limite de la courbe de variation on peut avoir des résultats négatifs
  return forvib;
}
// ********************************** FIN ***************************************
